/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.cache;

import lombok.extern.slf4j.Slf4j;
import net.lensfrex.dscape.configure.GlobalConstant;
import net.lensfrex.dscape.dao.entity.UserBasic;
import net.lensfrex.dscape.utils.ObjectJsonSerializer;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class UserBasicCache {

    private static final String USER_INFO_CACHE_KEY = GlobalConstant.USER_INFO_CACHE_KEY;

    private static final String USERNAME_UID_CACHE_KEY = GlobalConstant.USERNAME_UID_CACHE_KEY;

    private final ObjectJsonSerializer objectJsonSerializer;

    private final RedisTemplate<String, String> redis;

    public UserBasicCache(ObjectJsonSerializer objectJsonSerializer, RedisTemplate<String, String> redis) {
        this.objectJsonSerializer = objectJsonSerializer;
        this.redis = redis;
    }

    public UserBasic getUserByUid(String uid) {
        String cacheData = (String) redis.opsForHash().get(USER_INFO_CACHE_KEY, uid);
        if (cacheData != null) {
            return objectJsonSerializer.deserialize(cacheData, UserBasic.class);
        } else {
            log.debug("缓存中没有用户信息");
            return null;
        }
    }

    public UserBasic getUserByUserName(String username) {
        String uid = (String) redis.opsForHash().get(USERNAME_UID_CACHE_KEY, username);
        if (uid != null) {
            return this.getUserByUid(uid);
        } else {
            log.debug("缓存中没有用户名和uid的对应信息");
            return null;
        }
    }

    public void save(UserBasic userBasic) {
        redis.opsForHash().put(USER_INFO_CACHE_KEY, userBasic.getUid(), objectJsonSerializer.serialize(userBasic));
        redis.opsForHash().put(USERNAME_UID_CACHE_KEY, userBasic.getUserName(), userBasic.getUid());
    }

    public void clear(String uid) {
        redis.opsForHash().delete(USER_INFO_CACHE_KEY, uid);
    }
}
