/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.web.controllers.user.admin;

import cn.dev33.satoken.stp.StpUtil;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.dscape.dto.response.general.Response;
import net.lensfrex.dscape.dto.response.general.ResponseCode;
import net.lensfrex.dscape.enums.user.UserRoleEnum;
import net.lensfrex.dscape.exception.GlobalException;
import net.lensfrex.dscape.web.service.user.admin.AdminService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/user/admin")
public class AdminController {

    private final AdminService adminService;

    public AdminController(AdminService adminService) {
        this.adminService = adminService;
    }

    @PostMapping(value = "/add", produces = "application/json")
    public Response<Object> addUser(@RequestBody String body,
                            @RequestHeader String token) {
        return Response.error(ResponseCode.API_NOT_IMPLEMENT);
    }

    @PostMapping(value = "/modifyStatus", produces = "application/json")
    public Response<Object> modifyStatus(@RequestParam String uid, @RequestParam int status) {
        if (!StpUtil.hasRole(String.valueOf(UserRoleEnum.ADMIN.getCode()))) {
            throw new GlobalException(ResponseCode.PERMISSION_DENIED);
        }

        switch (status) {
            case 0:
            case 1:
                adminService.modifyUserStatus(uid, status);
                break;
            case 2:
                adminService.deleteUser(uid);
                break;
            case 3:
                adminService.addUserToBlackList(uid);
                break;
            default:
                throw new GlobalException(ResponseCode.PARAM_WRONG);
        }

        return Response.success();
    }

    @GetMapping(value = "/inviteCode", produces = "application/json")
    public Response<List<String>> generateInviteCodes(@RequestParam(required = false, defaultValue = "-1") long expired,
                                                      @RequestParam(required = false, defaultValue = "1") int count) {
        if (!StpUtil.hasRole(String.valueOf(UserRoleEnum.ADMIN.getCode()))) {
            throw new GlobalException(ResponseCode.PERMISSION_DENIED);
        }

        if (count > 1024) {
            throw new GlobalException(ResponseCode.INVALID_REQUEST);
        }

        List<String> result = adminService.generateInviteCode(expired, count);

        return Response.success(result);
    }
}