/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.web.service.user.admin;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import net.lensfrex.dscape.cache.InviteCodeService;
import net.lensfrex.dscape.cache.UserBasicCache;
import net.lensfrex.dscape.dao.entity.BlackList;
import net.lensfrex.dscape.dao.entity.UserBasic;
import net.lensfrex.dscape.dao.service.BlackListService;
import net.lensfrex.dscape.dao.service.UserBasicService;
import net.lensfrex.dscape.enums.user.BlackListTypeEnum;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class AdminService {
    private final RedisTemplate<String, String> redis;

    private final UserBasicService userBasicService;

    private final BlackListService blackListService;

    private final InviteCodeService inviteCodeService;

    private final UserBasicCache userBasicCache;

    public AdminService(RedisTemplate<String, String> redis,
                        UserBasicService userBasicService,
                        BlackListService blackListService,
                        InviteCodeService inviteCodeService,
                        UserBasicCache userBasicCache) {

        this.redis = redis;
        this.userBasicService = userBasicService;
        this.blackListService = blackListService;
        this.inviteCodeService = inviteCodeService;
        this.userBasicCache = userBasicCache;
    }

    public List<String> generateInviteCode(long expired, int count) {
        return inviteCodeService.generateInviteCode(expired, count);
    }

    public void modifyUserStatus(String uid, int status) {
        UpdateWrapper<UserBasic> wrapper = new UpdateWrapper<>();
        wrapper.lambda()
                .eq(UserBasic::getUid, uid)
                .eq(UserBasic::getSuperior, StpUtil.getLoginIdAsString())
                .set(UserBasic::getStatus, status);

        this.clearUserBasicCache(uid);
        userBasicService.update(wrapper);
    }

    public void addUserToBlackList(String uid) {
        BlackList blackList = new BlackList();
        blackList.setData(uid);
        blackList.setType(BlackListTypeEnum.UID.getCode());

        blackListService.save(blackList);
    }

    public void deleteUser(String uid) {
        UpdateWrapper<UserBasic> wrapper = new UpdateWrapper<>();
        wrapper.lambda()
                .eq(UserBasic::getUid, uid)
                .set(UserBasic::getIsDeleted, 1);

        this.clearUserBasicCache(uid);
        userBasicService.update(wrapper);
    }

    private void clearUserBasicCache(String uid) {
        userBasicCache.clear(uid);
    }
}